#    This is a part of the "Prototype Tagger", an unsupervised POS tagger developed
#    in the Hebrew University by Omri Abend, Roi Reichart and Ari Rappoport.
#    Copyright (C) 2010  Omri Abend, Roi Reichart and Ari Rappoport
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    If you have any questions, feel free to contact us in:
#    omria01@cs.huji.ac.il, roiri@cs.huji.ac.il
#
#    Please read the README file for more details.
#
#!/usr/local/bin/python
import utils, re, pickle

# Runs the first stage of the algorithm. (coordinate clustering and prototype detection stage)
class first_stage_driver:
    
    def __init__(self, settings):
        self._settings = settings
    
    def run(self):
    
        dict_filename = self._settings.get_string("LEXICON_FILE")
        thresh = self._settings.get_float("ALPHA")
        N_categories = self._settings.get_int("K_CATEGORIES")
        max_num_of_iterations = self._settings.get_int("NUMBER_ITERATIONS")
        morpho_clustering_file = self._settings.get_string("MORPHO_CLUSTERING_FILE")
        freq_threshold = self._settings.get_int("FREQUENCY_THRESHOLD")
        category_min_size = self._settings.get_int("MIN_SIZE_PER_CATEGORY")
        
        iteration_counter = 1
        number_of_clustering_iterations = 1
        
        dict_f = open(dict_filename)
        vector_dict, freq_dict, temp_line = utils.read_dictionary(dict_f, None, None, freq_threshold)
        dict_f.close()
        
        while number_of_clustering_iterations > 0 and iteration_counter <= max_num_of_iterations:
            print "Clustering iteration #" + str(iteration_counter)
            
            initial_cluster_lists = self.read_morpho_clustering(morpho_clustering_file, vector_dict.keys(), vector_dict, thresh, freq_dict)
            cluster_lists, number_of_clustering_iterations = utils.average_link_clustering(vector_dict, thresh, initial_cluster_lists, freq_dict)

            # Updating the dictionaries
            if iteration_counter < max_num_of_iterations:
                utils.update_dictionary_by_clusters(vector_dict, cluster_lists)
            
            iteration_counter += 1
       
        cluster_lists_original = get_clone(cluster_lists)
        
        # Removing the smaller than threshold clusters
        utils.remove_small_clusters(cluster_lists, category_min_size)
        
        # Extracting prominent categories
        
        cos_dict = utils.get_cos_dict_clusters(cluster_lists, vector_dict)
        prominent_clusters_list = utils.prominent_clusters(cluster_lists, freq_dict, vector_dict, 
                                                     cos_dict, N_categories)
        
        prototypes = [cluster_lists[prominent_cluster] for prominent_cluster in prominent_clusters_list]
        
        return prototypes, cluster_lists_original
    # End of method
    
    # Reads the morphological clustering and returns the data structure containing
    # that clustering. 
    # word_list - contains the list of words that appear in the dictionary. Every word
    #             which is not included in the morpho clustering will be set to a singleton.
    # vector_dict - maps the words to vectors
    def read_morpho_clustering(self, pickle_file_name, word_list, vector_dict, thresh, freq_dict):
        pickle_file = open(pickle_file_name)
        morpho_clustering = pickle.load(pickle_file)
        pickle_file.close()
        
        induced_clustering = dict()
        clustered_words = []
        for cluster_name in morpho_clustering.keys():
            cluster = morpho_clustering[cluster_name]
            reduced_cluster = []
            for word in cluster:
                if word in word_list:
                    reduced_cluster.append(word)
            
            cur_cluster_vector_dict = dict([(x,vector_dict[x]) for x in reduced_cluster])
            if len(cur_cluster_vector_dict.keys()) < 2:
                continue
            else:
                sub_clusters, number_of_iterations = \
                        utils.average_link_clustering(cur_cluster_vector_dict, thresh)
            
            for sub_cluster in sub_clusters.items():
                induced_clustering[sub_cluster[0]] = sub_cluster[1]
                clustered_words.extend(sub_cluster[1])
            
        for word in word_list:
            if not word in clustered_words:
                induced_clustering[word] = [word]
        
        return induced_clustering
    # End of method
    
# Returns a clone of the clustering
def get_clone(cluster_lists):
    if cluster_lists == None:
        return None
    output = dict()
    for cluster_name in cluster_lists.keys():
        new_list = [x for x in cluster_lists[cluster_name]]
        output[cluster_name] = new_list
    return output
# End of method
