#    This is a part of the "Prototype Tagger", an unsupervised POS tagger developed
#    in the Hebrew University by Omri Abend, Roi Reichart and Ari Rappoport.
#    Copyright (C) 2010  Omri Abend, Roi Reichart and Ari Rappoport
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    If you have any questions, feel free to contact us in:
#    omria01@cs.huji.ac.il, roiri@cs.huji.ac.il
#
#    Please read the README file for more details.
#
#!/usr/local/bin/python
import sys
# Implements a matrix

class matrix():
    
    def __init__(self, n):
        self._matrix =  [[0.0 for c in range(l)] for l in range(n)]
        self._active_indices = [1 for i in range(n)]
        self._n = n
    
    def put(self, i, j, val):
        if self._active_indices[i] == 1 and self._active_indices[j] == 1 and i != j:
            if i < j:
                temp = i
                i = j
                j = temp
            self._matrix[i][j] = val
    
    def max_matrix(self):
        max_val = -100
        max_key = None
        
        for i in range(1,len(self._matrix)):
            if self._active_indices[i] == 0:
                continue
            cur_max = max(self._matrix[i])
            if cur_max > max_val:
                max_key = (i, self._matrix[i].index(cur_max))
                max_val = cur_max
        
        return max_key, max_val
    
    def get(self, i, j):
        if self._active_indices[i] == 0 or self._active_indices[j] == 0 or i == j:
            return 0
        if i < j:
            temp = i
            i = j
            j = temp
        return self._matrix[i][j]
        
    # Merges the two rows and the two columns with coefficients alpha1 and alpha2 
    def merge_rows_columns(self, ind1, ind2, alpha1, alpha2):
        if self._active_indices[ind1] == 0 or self._active_indices[ind2] == 0 or ind1 == ind2:
            sys.stderr.write("Error in matrix class")
            return
        row1 = [self.get(ind1, x) for x in range(self._n)]
        row2 = [self.get(ind2, x) for x in range(self._n)]
        for index in range(self._n):
            self.put(ind1, index, alpha1*row1[index] + alpha2*row2[index])
        for index in range(self._n):
            self.put(ind2, index, 0)
        self._active_indices[ind2] = 0

    def size(self):
        return self._n
