#    This is a part of the "Prototype Tagger", an unsupervised POS tagger developed
#    in the Hebrew University by Omri Abend, Roi Reichart and Ari Rappoport.
#    Copyright (C) 2010  Omri Abend, Roi Reichart and Ari Rappoport
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    If you have any questions, feel free to contact us in:
#    omria01@cs.huji.ac.il, roiri@cs.huji.ac.il
#
#    Please read the README file for more details.
#
#!/usr/local/bin/python
import sys, re, pickle, first_stage_driver, second_stage_driver, morfessor_wrapper, lexicon_extractor, os, settings_loader
from parameters_dast import *

# The alpha, beta which were specified in the paper are actually 1-alpha and 1-beta which
# are specified by this code. This method fixes that.
def fix_alpha_beta(settings):
    alpha = settings.get_float("ALPHA")
    beta = settings.get_float("BETA")
    
    alpha = 1 - alpha
    beta = 1 - beta
    
    settings.set_string("ALPHA", str(alpha))
    settings.set_string("BETA", str(beta))
    
# END OF METHOD

# Runs the prototype tagger.
if len(sys.argv) != 2:
    print 'Usage: run_prototype_tagger <settings file>'
    sys.exit(-1)

settings = settings_loader.load_settings(sys.argv[0], sys.argv[1])

fix_alpha_beta(settings)

extractor = lexicon_extractor.lexicon_extractor(settings)
extractor.run()

wrapper = morfessor_wrapper.morfessor_wrapper(settings)
wrapper.run()

driver = first_stage_driver.first_stage_driver(settings)
prototypes, cluster_lists_original = driver.run()

if len(prototypes) == 0:
    sys.stderr.write("The parameters of frequency threshold and the minimum size per category is too high.\n")
    sys.stderr.write("Analysis did not yield any prototypes.\n")
    sys.exit(-1)

second_driver = second_stage_driver.second_stage_driver(settings)
type_mapping, freq_dict = second_driver.run(prototypes, cluster_lists_original)

# Output.
type_mapping_f = open(settings.get_string("TYPE_MAPPING_FILE"),"w")
for word in type_mapping.keys():
    type_mapping_f.write(word[1:-1].upper()+" "+type_mapping[word]+" "+str(int(freq_dict[word]))+"\n")
type_mapping_f.close()

print 'Done.'

